/*
 * ScenarioGraph.java
 *
 * Created on 31 marzec 2004, 11:37
 */

package pl.psnc.vlab.graph.model;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import javax.imageio.ImageIO;
import javax.swing.ToolTipManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.JGraph;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.Edge;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.Port;
import org.jgraph.util.JGraphUtilities;
import pl.psnc.vlab.dev.DeviceDocument;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.ScenarioGraphEditor;
import pl.psnc.vlab.graph.ui.ScenarioGraphUI;
import pl.psnc.vlab.graph.util.GraphHelper;
import pl.psnc.vlab.graph.vertex.ImageCell;
import pl.psnc.vlab.graph.view.ImageVertexView;
import pl.psnc.vlab.ssa.exception.SSAException;
import pl.psnc.vlab.ssa.exception.SSAExceptionCodes;
import pl.psnc.vlab.xml.DeviceFactory;




/**
 *
 * @author  Dominik Stokl0osa
 * @email  osa@man.poznan.pl
 */
public class ScenarioGraph extends JGraph{
    
    /** Instance of logger*/
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    public static String FILE_FORMAT_VERSION = "PAD-1.0";
    protected Hashtable writeProperties = new Hashtable();
    protected boolean pagevisible = false;
    
    /** Instance of Marquee Handler */
    private ScenarioMarqueeHandler marqueeHandler = null;
    
    /** Instance of Scenario Graph UI */
    //private static ScenarioGraphUI scenarioGraphUI = new ScenarioGraphUI();
    
    /** Instance of ScenarioGraphEditor */
    ScenarioGraphEditor scenarioGraphEditor = null;
    
    private ScenarioGraphSelectionListener scenGraphSelListener;
    
    
    private static BufferedImage original;
    private static BufferedImage rescaled;
    
    
    
    /** Creates a new instance of ScenarioGraph
     *  using the Model as its Data Source
     * @param scenarioGraphEditor
     * @param model
     * @throws SSAException
     */
    public ScenarioGraph(ScenarioGraphEditor scenarioGraphEditor, DefaultGraphModel model, GraphLayoutCache view) throws SSAException{
        super(model, view);
        try{
            this.scenarioGraphEditor = scenarioGraphEditor;
            this.marqueeHandler = new ScenarioMarqueeHandler(this);
            setMarqueeHandler( this.marqueeHandler );
            //set tooltip manager
            ToolTipManager.sharedInstance().registerComponent(this);
            
            original = ImageIO.read(getClass().getClassLoader().getResource("images/map/europa5.png"));
            rescaled = original;
            
            // Tell the Graph to Select new Cells upon Insertion
            // removed 5.10 setSelectNewCells(true);
            // Make Ports Visible by Default
            
            setPortsVisible(true);
            
            // Use the Grid (but don't make it Visible)
            setGridEnabled(true);
            // Set the Grid Size to 10 Pixel
            setGridSize(6);
            // Set the Tolerance to 2 Pixel
            setTolerance(2);
            //setForeground(Color.ORANGE);
            setHighlightColor(Color.orange);
            setHandleColor(Color.BLACK);
            setLockedHandleColor(Color.black);
            setGridColor(Color.orange);
            setAutoResizeGraph(true);            
            
            //add graph model listener
            getModel().addGraphModelListener(new ScenarioGraphModelListener(this));
            //add graph observer
            //removed 5.10 getGraphLayoutCache().addObserver( new ScenarioGraphViewListener(this));
            //tmb 5.10 getModel().addGraphModelListener(new ScenarioGraphViewListener(this));
            //add graph selection listener
            
            scenGraphSelListener = new ScenarioGraphSelectionListener(this);
            this.addGraphSelectionListener(scenGraphSelListener);
            
            
            
            addComponentListener(new java.awt.event.ComponentAdapter() {
                public void componentResized(java.awt.event.ComponentEvent evt) {
                    tranlateLocation();
                }
            });
            
            
            //set new RdsDocument instance
        }catch (Exception e){
            throw new SSAException(e.getMessage(), SSAExceptionCodes.GENERAL_EXCEPTION, e);
        }
        
    }
    
    
    private void tranlateLocation(){
        log.debug("<tranlateLocation>");
        Object vertexList[] = JGraphUtilities.getVertices(this, getAll());
        int size = vertexList != null ? vertexList.length : 0;
        log.debug("Size="+size);
        for (int i = 0; i < size; i++) {
            Object userObject = ( (DefaultGraphCell) vertexList[i]).getUserObject();
            int resId = ( (ScenarioCellObject) userObject).getResource().getId();
            try {
                DeviceDocument.Device device = DeviceFactory.getInstance().getDevice(resId);
                if(device == null){
                    log.debug("Dev is null");
                    continue;
                }
                Point2D p = ( (ScenarioGraphUI) getUI()).rescale(device.getLocation().getX(), device.getLocation().getY() );
                log.debug("New point="+p);                
                ImageVertexView cellView = (ImageVertexView) getGraphLayoutCache().getMapping(vertexList[i], false);        
                cellView.update(this, p);
            } catch (VlabException ex) {
                ex.printStackTrace();
                //ok to ignore
            }          
        } // end of for
       // repaint();
    }
    
    
    /**
     * Get the Scenario Graph Selection Listener
     * @return
     */
    public ScenarioGraphSelectionListener getScenarioGraphSelectionListener(){
        return scenGraphSelListener;
    }
    
    
    public ScenarioGraphEditor getGraphEditor(){
        return this.scenarioGraphEditor;
    }
    
    
    /**
     * Notification from the <code>UIManager</code> that the L&F has changed.
     * Replaces the current UI object with the latest version from the
     * <code>UIManager</code>. Subclassers can override this to support
     * different GraphUIs.
     * @see JComponent#updateUI
     *
     */
    public void updateUI() {
        setUI(new ScenarioGraphUI() );
        invalidate();
    }
    
    
    
    public void setPageVisible(boolean flag) {
        pagevisible = flag;
    }
    
    public boolean isPageVisible() {
        return pagevisible;
    }
    
    
    /**
     * Returns the writeProperties.
     * @return Hashtable
     */
    public Hashtable getWriteProperties() {
        return writeProperties;
    }
    
    /**
     * Sets the writeProperties.
     * @param writeProperties The writeProperties to set
     */
    public void setWriteProperties(Hashtable writeProperties) {
        this.writeProperties = writeProperties;
    }
    
    
    /** Gets all elements frotm the graph
     * @return
     */
    public Object[] getAll() {
        // removed Graph 10.1 return getDescendantList(getRoots());
        return JGraphUtilities.getAll(this);
    }
    
    
    /**
     * Get the list of resources described by the graph cells
     * @return
     */
    public ArrayList getAllResourcesDesc() {
        log.debug("<getAllResourcesDesc>");
        //get the list of cells
        Object[] cellList = getAll();
        int size = cellList.length;
        ArrayList result = new ArrayList(size);
        for(int i=0; i<size; i++){
            String type = GraphCellFactory.getType(cellList[i]);
            if(type.equals(GraphCellFactory.CELL_TYPE_IMAGE)){
                ScenarioCellObject cellObject = (ScenarioCellObject) ((ImageCell) cellList[i]).getUserObject();
                //removed 12.07.2004 Object resourceDesc = cellObject.getProperty(GraphKeys.PROPERTY_NODE_RESOURCE_DESC);
                Object resourceDesc = cellObject.getResource();
                if(resourceDesc != null){
                    result.add(resourceDesc);
                }
            }// end of if
        }
        log.debug("</getAllResourcesDesc>");
        return result;
    }
    
    
    public void update() {
        ScenarioGraphModel model =  (ScenarioGraphModel) this.getModel();
        //model.beginUpdate();
        getGraphLayoutCache().reload();
        
    }
    
    
    /* Sets the attributes of the selected cells. */
    public void setSelectionAttributes(AttributeMap map) {
        Object[] cells =DefaultGraphModel.getDescendants(getModel(), getSelectionCells()).toArray();
        //Filter ports out
        List list = new ArrayList();
        for (int i = 0; i < cells.length; i++){
            if (!isPort(cells[i]))
                list.add(cells[i]);
        }
        cells = list.toArray();
        
        map = (AttributeMap) map.clone();
        map.remove(GraphConstants.BOUNDS);
        map.remove(GraphConstants.POINTS);
        
        Map nested = new Hashtable();
        for (int i = 0; i < cells.length; i++)
            nested.put(cells[i], (AttributeMap) map.clone());
        getGraphLayoutCache().edit(nested, null, null, null);
    }
    
    
    public boolean isPort(Object object) {
        return (object instanceof Port);
    }
    
    public boolean isEdge(Object object) {
        return (object instanceof Edge);
    }
    
    /**
     * Tell whether the current graph has any child elements attached to him
     * @return
     */
    public boolean isEmpty(){
        Object res[] = getAll();
        if(res != null && res.length >0){
            return false;
        }else {
            return true;
        }
    }
    
    public String getToolTipText(java.awt.event.MouseEvent mouseEvent) {
        if(mouseEvent == null){
            return null;
        }
        // Fetch Cell under Mousepointer
        Object cell = getFirstCellForLocation(mouseEvent.getX(), mouseEvent.getY());
        if (cell != null) {
            // Convert Cell to String and Return
            ScenarioCellObject userObject = GraphHelper.getCellObject(cell);
            String res = userObject != null ? userObject.getToolTip() : null;
            return res;
        }
        return null;
    }
    
    
}
