/*
 * GraphHelper.java
 *
 * Created on 10 sierpie� 2004, 14:13
 */

package pl.psnc.vlab.graph.util;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultPort;
import org.jgraph.util.JGraphUtilities;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.graph.model.ScenarioCellObject;
import pl.psnc.vlab.graph.model.ScenarioGraph;
import pl.psnc.vlab.graph.vertex.ImageCell;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.rds.ElementGroupDocument;
import pl.psnc.vlab.rds.ResourceDocument;
import pl.psnc.vlab.rds.TabDocument;
import pl.psnc.vlab.ssa.gui.MainFrame;
import pl.psnc.vlab.ssa.gui.resource.PropertyDialogFactory;
import pl.psnc.vlab.ssa.gui.resource.pane.JResourcePropertyDialog;
import pl.psnc.vlab.ssa.util.ImageKeys;
import pl.psnc.vlab.tds.TypeDocument;
import pl.psnc.vlab.util.image.ImageBean;
import pl.psnc.vlab.util.image.ImageFactory;
import pl.psnc.vlab.util.swing.gui.Message;
import pl.psnc.vlab.xml.LinksFactory;
import pl.psnc.vlab.xml.TypeFactory;

/**
 *
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class GraphHelper {
    
    
    /** Logger instance */
    private static Log log = LogFactory.getLog("GraphHelper");
    
    
    /**
     * Get the vertex image icon for the type represented by
     * the given id. If no type is found the default image icon is
     * returned
     * @param typeId VLab resource type identifier
     * @return vertex image
     */
    public static ImageBean getResourceImage(int typeId){
        TypeDocument.Type type = TypeFactory.getResourceType(typeId);
        if(type == null){
            //tbd create default vertex image
            return ImageFactory.getImageBean(ImageKeys.DEAFULT_COMPUTE_BIG);
        }
        return ImageFactory.getImageBean(type.getImage().getVertex().getPath());
    }
    
    
    /**
     * Get the resource icon for the type represented by
     * the given id. If no type is found the default icon is
     * returned
     * @param typeId VLab resource type identifier
     * @return resource icon
     */
    public static ImageBean getResourceIcon(int typeId){
        TypeDocument.Type type = TypeFactory.getResourceType(typeId);
        if(type == null){
            //tbd create default vertex image
            return ImageFactory.getImageBean(ImageKeys.DEAFULT_COMPUTE_BIG);
        }
        return ImageFactory.getImageBean(type.getImage().getIcon().getPath());
    }
    
    /**
     * Get the generic type id for type given by the type id
     * @param typeId child type identifier
     * @return
     */
    public static int getParentTypeId(int typeId){
        TypeDocument.Type type = TypeFactory.getResourceType(typeId);
        if(type.getGenericTypeId() == 0){
            return type.getId();
        }
        return type.getGenericTypeId();
    }
    
    /**
     * Checks all the edges. If they are not valid - they are removed
     * @param graph
     */
    public static void validateEdges(ScenarioGraph graph){
        log.debug("<validateEdges>");
        //get edges list
        Object edges[] = JGraphUtilities.getEdges(graph);
        int size = edges != null ? edges.length : 0;
        for(int i=0; i<size; i++){
            Object src = JGraphUtilities.getSourceVertex(graph, edges[i]);
            Object dst = JGraphUtilities.getTargetVertex(graph, edges[i]);
            try{
                int srcId = ((ScenarioCellObject)((DefaultGraphCell) src).getUserObject()).getResource().getTypeId();
                int dstId = ((ScenarioCellObject)((DefaultGraphCell) dst).getUserObject()).getResource().getTypeId();
                if(!LinksFactory.canConnect(srcId, dstId) ){
                    //remove the edge
                    JGraphUtilities.removeCells(graph, new Object[] { edges[i] });
                }
            }catch(Exception e){
                continue;
            }
        }
        log.debug("</validateEdges>");
    }
    
    
    /**
     * Check wheter the specified graph has at least one occurrence of the initital mark.
     * The initial mark makes node to become an initial node. It is important during
     * the scenario deserialization process
     * @return
     * @param graph
     */
    public static boolean hasInitialNodes(ScenarioGraph graph){
        Object cells[] = JGraphUtilities.getAll(graph);
        int size = cells != null ? cells.length : 0;
        for(int i=0; i<size; i++){
            if(!JGraphUtilities.isVertex(graph, cells[i]) ){
                continue;
            }
            ScenarioCellObject cellObject  = (ScenarioCellObject) ((DefaultGraphCell) cells[i]).getUserObject();
            if(cellObject.isMarkedInitial()){
                return true;
            }
        }//end of for
        return false;       //no initial marks has been found
    }
    
    
    /**
     * Test whether the given cell is generic type
     * @param cell graph cell
     * @return
     */
    public static boolean isGeneric(Object cell){
        if( !(cell instanceof ImageCell)){
            return false;
        }
        ScenarioCellObject cellObject = getCellObject(cell);
        boolean res = cellObject != null ? cellObject.getType().isGenericType() : false;
        return res;
    }
    
    
    /**
     * Test whether the given cell is vertex type
     * @param cell graph cell
     * @return true/false
     */
    public static boolean isVertex(Object cell){
        if(cell == null){
            return false;
        }
        boolean res = cell instanceof ImageCell ? true : false;
        return res;
    }
    
    /**
     * Test whether the given cell is edge
     * @param cell graph cell
     * @return true/false
     */
    public static boolean isEdge(Object cell){
        if(cell == null){
            return false;
        }
        boolean res = cell instanceof DefaultEdge ? true : false;
        return res;
    }
    
    /**
     * Test whether the given cell is port
     * @param cell graph cell
     * @return true/false
     */
    public static boolean isPort(Object cell){
        if(cell == null){
            return false;
        }
        boolean res = cell instanceof DefaultPort ? true : false;
        return res;
    }
    
    
    /**
     * Get the number of verticles with defined resource.
     * @param graph
     * @return
     */
    public static int countConcreteVerticles(ScenarioGraph graph){
        Object verticles[] = JGraphUtilities.getVertices(graph, JGraphUtilities.getAll(graph));
        int size = verticles != null ? verticles.length : 0;
        int count = 0;
        for(int i=0; i<size; i++){
            if( !isGeneric((DefaultGraphCell) verticles[i])){
                count++;
            }
        }
        return count;
    }
    
    public static JResourcePropertyDialog validateDates(ScenarioCellObject cellObject, ResourceDocument.Resource resource) throws VlabException{
        log.debug("<validateDates>");
        if(resource == null){
            log.debug("</validateDates:null.I>");
            return null;
        }
        JResourcePropertyDialog d = (JResourcePropertyDialog) PropertyDialogFactory.getResourcePropertyDialog(MainFrame.getInstance(), true, resource);
                //null;
        int counter = 0;
        ResourceDocument.Resource.Tabs tabs = resource.getTabs();
        TabDocument.Tab[] tabList = tabs.getTabArray();
        int tabSize = tabList != null ? tabList.length : 0;
        for(int k=0; k<tabSize; k++){
            ElementDocument.Element elemArray[] = tabList[k].getElementArray();
            int elemSize = elemArray != null ? elemArray.length : 0;
            for(int z=0; z<elemSize; z++){
                if( elemArray[z].getComponent().getId() == 6 || elemArray[z].getName().equalsIgnoreCase("expLength") ){
                    if(elemArray[z].getOut().sizeOfItemArray() <= 0){
                        //add component
                        /*
                        if(d == null){
                            d =  (JResourcePropertyDialog) PropertyDialogFactory.getResourcePropertyDialog(MainFrame.getInstance(), true, resource);
                        }*/
                        d.requestFocus(elemArray[z].getName());
                        counter++;
                    }
                }
            }//end of z for
            //group elements
            ElementGroupDocument.ElementGroup elemGArray[] = tabList[k].getElementGroupArray();
            int elemGSize = elemGArray != null ? elemGArray.length : 0;
            for(int z=0; z<elemGSize; z++){
                //elements in groups
                elemArray = elemGArray[z].getElementArray();
                elemSize = elemArray != null ? elemArray.length : 0;
                for(int j=0; j<elemSize; j++){
                    if( elemArray[j].getComponent().getId() == 6 || elemArray[j].getName().equalsIgnoreCase("expLength")){
                        if(elemArray[j].getOut().sizeOfItemArray() <= 0){
                            /*if(d == null){
                                d =  (JResourcePropertyDialog) PropertyDialogFactory.getResourcePropertyDialog(MainFrame.getInstance(), true, resource);
                            }*/
                            d.requestFocus(elemArray[j].getName());
                            counter++;
                        }
                    }
                }//end of j for
            }//end of z for
        }
        
        if(counter >0){
            //show info and property dialog
            Message.showInfoMessage(MainFrame.getInstance(), "The Date Field has not been set. Please enter the valid date before submitting the scenario.\n " +
                    "All empty date fields will be marked red.", Message.MESSAGE_TYPE_INF);
            //d.show();
            //cellObject.setResource(d.getResource());
            log.debug("</validateDates:dialog>");
            return d;
        }
        log.debug("</validateDates:null.II>");
        return null;
    }
    
    
    /**
     * Get the user object from the given cell
     * @param cell cell object
     * @return user object or null
     */
    public static ScenarioCellObject getCellObject(Object cell){
        if(cell == null){
            return null;
        }
        DefaultGraphCell defaultGraphCell = (DefaultGraphCell) cell;
        Object userObject = defaultGraphCell.getUserObject();
        if(userObject == null){
            return null;
        }
        return (ScenarioCellObject) userObject;
    }
}
