/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import pl.psnc.vlab.ssa.gui.resource.ComponentBuilder;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.rds.ElementDocument.Element;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.ssa.gui.resource.model.ModelDataComboBoxModel;
import pl.psnc.vlab.xml.ComponentFactory;


/**
 * This class represents the JComboBox component. Having the element
 * details specyfied the JComboBox component will be configured properly
 * TBD add the possibility of adding elements to combo
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public class ComboBoxElement extends ElementComponent {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public ComboBoxElement(Element resourceElement) {
        super(resourceElement);
        elementComponent = new JComboBox();
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        //set editable
        setEditable(isEditable());
        //set model
        installComboModel();
        //set selected index
        setSelectedIndex(getSelectedIndex());
        //set selected item
        setSelectedItem(getSelectedItem());
    }
    
    
    /**
     *  Set the model for this component
     */
    private  void installComboModel(){
        log.debug("<installComboModel>");
        ModelDataComboBoxModel model = new ModelDataComboBoxModel(doCast(), getModelItems() );
        setModel(model);
        log.debug("</installComboModel>");
    }
    
    
    private void setComponentValue(){
        log.debug("<getOutItems>");
        ElementDocument.Element.Out out = resourceElement.getOut();
        int itemsCount = out.sizeOfItemArray();
        log.debug("Items count:"+itemsCount);
        if(itemsCount != 1){
            return;
        }
        ItemDocument.Item item = out.getItemArray()[0];
        log.debug("Got item:"+item.getStringValue() );
        doCast().setSelectedItem(item.getStringValue());
        log.debug("</getOutItems>");
    }
    
    
    /**
     * Notifies the observers that they should update their data
     * @param resource the resource description
     */
    public void update() {
        log.debug("<update>");
        //get selected object
        ItemDocument.Item selectedItem = ( (ModelDataComboBoxModel)this.doCast().getModel()).getSelectedItemAsItem();
        if(selectedItem == null){
            log.debug("<update: NOTHING SELECTED>");
            return;
        }
        ElementDocument.Element.Out out = resourceElement.getOut();
        out.setItemArray(new ItemDocument.Item[]{selectedItem} );
        log.debug("</update>");
    }
    
    
    /**
     * Tells whether the component will be editable or not.
     * Set to true by default
     * @return
     */
    public boolean isEditable(){
        if(resourceElement.getComponent().isSetEditable()){
            return resourceElement.getComponent().getEditable();
        } else if(resourceElementType.isSetEditable()){
            return resourceElementType.getEditable();
        }
        return true;
    }
    
    /**
     * Sets the specified boolean to indicate whether or not this TextComponent
     * should be editable. A PropertyChange event ("editable") is fired when the state is changed.
     * @param editable
     */
    public void setEditable(boolean editable){
        //set resource as enabled
        resourceElement.getComponent().setEditable(editable);
        //set component as enabled
        doCast().setEditable(editable);
    }
    
    
    
    /**
     * Selects the item at index anIndex.
     * @param anIndex an integer specifying the list item to select,
     * where 0 specifies the first item in the list and -1 indicates no selection
     */
    public void setSelectedIndex(int anIndex){
        try{
            doCast().setSelectedIndex(anIndex);
        } catch(IllegalArgumentException e){
            log.error("Setting selected index to :"+anIndex+" failed");
        }
    }
    
    
    /**
     * Returns the first item in the list that matches the given item.
     * The result is not always defined if the JComboBox  allows selected items
     * that are not in the list. Returns -1 if there is no selected item or
     * if the user specified an item which is not in the list.
     * @return an integer specifying the currently selected list item,
     * where 0 specifies the first item in the list; or -1 if no item is selected or if the currently selected item is not in the list
     */
    public int getSelectedIndex(){
        return resourceElement.getComponent().isSetSelectedIndex() ? resourceElement.getComponent().getSelectedIndex() : 0;
    }
    
    
    
    /**
     * Sets the selected item in the combo box display area to the object in the argument.
     * If anObject is in the list, the display area shows anObject selected.
     * If anObject is not in the list and the combo box is uneditable, it will not
     * change the current selection. For editable combo boxes, the selection will
     * change to anObject.  If this constitutes a change in the selected item,
     * ItemListeners added to the combo box will be notified with one or two ItemEvents.
     * If there is a current selected item, an ItemEvent will be fired and the state
     * change will be ItemEvent.DESELECTED. If anObject is in the list and is not currently
     * selected then an ItemEvent will be fired and the state change will be ItemEvent.SELECTED.
     * ActionListeners added to the combo box will be notified with an ActionEvent when
     * this method is called.
     * @param anObject the list object to select; use null to clear the selection
     */
    public void setSelectedItem(Object anObject){
        if(anObject == null){
            return;
        }
        try{
            doCast().setSelectedItem(anObject);
        }catch(Exception e){
            log.error("Selecting item failed :"+e.getMessage());
        }
    }
    
    
    /**
     * Returns the current selected item
     * @return  If the combo box is editable, then this value may not have been added to the combo box with addItem, insertItemAt or the data constructors.
     */
    public Object getSelectedItem(){
        ItemDocument.Item outItems[] = getOutItems();
        int itemsCount = outItems != null ? outItems.length : 0;
        if(itemsCount == 0){
            return null;
        }
        return outItems[0].getStringValue();
    }
    
    /**
     * Sets the data model that the JComboBox uses to obtain the list of items.
     * @param aModel the ComboBoxModel that provides the displayed list of items
     */
    public void setModel(ComboBoxModel aModel){
        doCast().setModel(aModel);
    }
    
    
    
    
    /**
     * Cast the element component to the JComboBox interface
     * @return element component as an JComboBox
     */
    private JComboBox doCast(){
        return (JComboBox) elementComponent;
    }
}
