/*
 * TextFieldElement.java
 *
 * Created on 17 czerwiec 2004, 10:53
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import pl.psnc.vlab.ssa.gui.resource.pane.DatePane;
import pl.psnc.vlab.ssa.util.component.ComponentKeys;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.management.PropertiesFactory;
import pl.psnc.vlab.ssa.util.Keys;
import pl.psnc.vlab.ssa.management.SystemConfigurationFactory;
import pl.psnc.vlab.ssa.management.SystemConfigurationValue;
import pl.psnc.vlab.util.DateTools;
//schema
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ItemDocument;
import pl.psnc.vlab.cds.ComponentDocument;
import pl.psnc.vlab.ssa.gui.MainFrame;

/**
 * This class represents the Date component. Having the element
 * details specyfied the Date component will be configured properly
 * <br> NOTE: The date and time are stored in the Vlab Internal representation,
 * which is the form of the time stamp yyyyMMddHHmmSS
 * @author Dominik Stoklosa
 * @email osa@man.poznan.pl
 */
public class DateElement extends ElementComponent {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    private static final String DEADLINE = "deadlineValue";
    private static final String FROM = "TimeFromValue";
    private static final String TO   = "TimeToValue";
    
    /** An instance of the date tools */
    private DateTools dateTools = null;
    
    /** Date field */
    public final static int DATE = 0;
    /** Time field */
    public final static int TIME = 1;
        
    
    /**
     * Creates a new instance of TextFieldElement, with the
     * element details specyfied by the resource element object
     * @param resourceElement
     */
    public DateElement(ElementDocument.Element resourceElement) {
        super(resourceElement);                        
        SystemConfigurationValue sysConfValue = SystemConfigurationFactory.getInstance();
        dateTools = new DateTools(sysConfValue.getCurrentLocale(),getDateFormat(), getTimeFormat());
        sysConfValue = null;
        //initialize the date component
        initComponent();
        initElement();
    }
    
    /**
     * Sets the element properties
     */
    protected void initElement(){
        //set element name
        String name = getComponentName();
        log.debug("Name:"+name);
        this.elementComponent.setName(name);
        //set font
        this.elementComponent.setFont(ComponentKeys.RESOURCE_VALUE_FONT);
        //set tooltip
        this.elementComponent.setToolTipText(getToolTipText());
        //set enabled
        this.elementComponent.setEnabled( isEnabled());
        //set visible
        this.elementComponent.setVisible(isVisible());
        
    }
    
    
    /**
     * Get the concrete component
     */
    public javax.swing.JComponent getConcreteJavaComponent() {
        return this.elementComponent;
    }
    
    
    
    /**
     * Notifies the observer that he has to update his data
     */
    public void update() {        
        setEnabled( this.elementComponent.isEnabled());
        String dateValue = doCast().getSelectedDate();
        String timeValue = doCast().getSelectedTime();
        log.debug("<update:"+dateValue+ " " +timeValue+">");
        
        ElementDocument.Element.Out out = resourceElement.getOut();
        int itemSize = out.sizeOfItemArray();
        if(itemSize > 1){
            for(int i=itemSize-1; i>0; i--){
                out.removeItem(i);
            }
        }
        ItemDocument.Item outItem = null;
        if(itemSize == 0){
            outItem = out.addNewItem();
        } else {
            outItem = out.getItemArray(0);
        }
        outItem.setId(0);
        if(isDateComponentVisible() && !isTimeComponentVisible()){
            String date = getDate(PropertiesFactory.getVlabInternalDateFormat(), getDateFormat(), dateValue );
            log.debug("Date-"+date);
            outItem.setStringValue(date);
        } else if(isTimeComponentVisible() && !isDateComponentVisible() ){
            String time = getDate(PropertiesFactory.getVlabInternalTimeFormat(), getTimeFormat(), timeValue );
            log.debug("Time-"+time);
            outItem.setStringValue(time);
        } else {
            String dateTime = dateTools.convertToFormat(PropertiesFactory.getVlabInternalDateTimeFormat(), dateValue, timeValue);
            log.debug("DateTime="+dateTime);
            outItem.setStringValue(dateTime);
        }
        log.debug("</update>");
    }
    
    /** Initialize the Date component */
    private void initComponent(){
        ItemDocument.Item items[] = getOutItems();
        String dates[] = getInitialDates(items);
        if(!isDateComponentVisible()){
            log.debug("Date part not visible");
            dates[DATE] = null;
        }
        
        if(!isTimeComponentVisible()){
            log.debug("Time part not visible");
            dates[TIME] = null;
        }
        SystemConfigurationValue sysConfValue = SystemConfigurationFactory.getInstance();
        this.elementComponent = new DatePane(sysConfValue.getCurrentLocale(),
        getDateFormat(), dates[DATE], getTimeFormat(), dates[TIME]);
        sysConfValue = null;
    }
    
    
    /**
     * Get the date format
     * @return
     */
    private String getDateFormat(){
        if(resourceElement.getComponent().isSetDateFormat()){
            return resourceElement.getComponent().getDateFormat();
        }
        if(resourceElementType.isSetDateFormat()){
            return resourceElementType.getDateFormat();
        }
        return Keys.DATE_FORMAT;
    }
    
    /**
     * Get the time format
     * @return
     */
    private String getTimeFormat(){
        if(resourceElement.getComponent().isSetTimeFormat()){
            return resourceElement.getComponent().getTimeFormat();
        }
        if(resourceElementType.isSetTimeFormat()){
            return resourceElementType.getTimeFormat();
        }
        return Keys.TIME_FORMAT;
    }
    
    /**
     * Specyfies whether the date component should be visible
     * @return
     */
    private boolean isDateComponentVisible(){
        int partValue = -1;
        if(resourceElement.getComponent().isSetDateParts()){
            partValue = resourceElement.getComponent().getDateParts().intValue();
        } else if(resourceElementType.isSetDateParts()){
            partValue = resourceElementType.getDateParts().intValue();
        }
        log.debug("Date part="+partValue);
        if(partValue != ComponentDocument.Component.DateParts.TIME.intValue()){
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Specyfies whether the time component should be visible
     * @return
     */
    private boolean isTimeComponentVisible(){
        int partValue = -1;
        if(resourceElement.getComponent().isSetDateParts()){
            partValue = resourceElement.getComponent().getDateParts().intValue();
        } else if(resourceElementType.isSetDateParts()){
            partValue = resourceElementType.getDateParts().intValue();
        }
        log.debug("Date part="+partValue);
        if(partValue != ComponentDocument.Component.DateParts.DATE.intValue()){
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Convert the date and time from the vlab internal representations
     * and returns them seperate as date and time accoridng to the specyfies
     * formats.
     * @param items
     * @return
     */
    private String[] getInitialDates(ItemDocument.Item[] items){
        log.debug("<getInitialDates>");
        int size = items != null ? items.length : 0;
        String dates[] = new String[2];
        if(size == 0){
            Date now = MainFrame.getInstance().getStatusBar().getCurrentDate();
            if(getComponentName().equalsIgnoreCase(DEADLINE) ){
                log.debug(" ----  DEADLINE ------");
                now = dateTools.addMonth(now, 1);
            } else if(getComponentName().endsWith(FROM)){
                log.debug(" ----  FROM ------");
                now = dateTools.add(now, Calendar.MINUTE,  5);
            } else if(getComponentName().endsWith(TO)){
                log.debug(" ----  TO ------");
                now = dateTools.add(now, Calendar.MINUTE,  25);
            }
            dates[DATE] = dateTools.getDateAsString(now);
            dates[TIME] = dateTools.getTimeAsString(now);
            log.debug("</getInitialDates:no initial date>");
            return dates;
        }
        
        
        
        if(isDateComponentVisible() && !isTimeComponentVisible()){
            dates[DATE] = getDate(getDateFormat(), PropertiesFactory.getVlabInternalDateFormat(), items[0].getStringValue());
            dates[TIME] = null;
            return dates;
        }
        
        if(isTimeComponentVisible() && !isDateComponentVisible()){
            dates[DATE] = null;
            dates[TIME] = getDate(getTimeFormat(), PropertiesFactory.getVlabInternalTimeFormat(), items[0].getStringValue());
            return dates;
        }
        
        DateTools internalDateTools = new DateTools(Locale.getDefault(), PropertiesFactory.getVlabInternalDateFormat(), null);
        Date outDate = internalDateTools.getDateFromString(items[0].getStringValue());
        log.debug("Internal date-"+outDate);
        dates[DATE] = getDate(getDateFormat(), PropertiesFactory.getVlabInternalDateTimeFormat(), items[0].getStringValue());
        dates[TIME] = getDate(getTimeFormat(), PropertiesFactory.getVlabInternalDateTimeFormat(), items[0].getStringValue());
        log.debug("</getInitialDates:"+dates[DATE]+"|"+dates[TIME]);
        return dates;
    }
    
    private String getDate(String newPattern, String curPattern, String value){
        DateTools internalDateTools = new DateTools(Locale.getDefault(), curPattern, null);
        Date outDate = internalDateTools.getDateFromString(value);
        //set new pattern
        internalDateTools = new DateTools(Locale.getDefault(), newPattern, null);
        return internalDateTools.getDateAsString(outDate);
    }
    

    /**
     * Cast the element component to the DatePane interface
     * @return element component as an DatePane
     */
    private DatePane doCast(){
        return (DatePane) elementComponent;
    }
    
}
