/*
 * ElementFactory.java
 *
 * Created on 17 czerwiec 2004, 11:05
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import java.awt.Color;
import pl.psnc.vlab.rds.ConstraintsDocument;
import pl.psnc.vlab.ssa.util.component.ComponentKeys;
import java.awt.Font;
import java.util.ArrayList;
import javax.swing.JComponent;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.cds.ExternalDocument;
import pl.psnc.vlab.management.session.SessionFactory;
import pl.psnc.vlab.ssa.gui.resource.model.ResourceElementObserver;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.values.management.SessionValue;
import pl.psnc.vlab.xml.ComponentFactory;
import pl.psnc.vlab.rds.ElementDocument;
import pl.psnc.vlab.cds.ComponentDocument;
import pl.psnc.vlab.cds.ItemDocument;


/**
 * The abstract super class for building the
 * java components from the resource elements.
 * It contains a set of common util methods i.e. for setting
 * the java component properties (name, tooltip, etc.)
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class ElementComponent implements ResourceElementObserver {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Stores the value of the resource element */
    protected ElementDocument.Element resourceElement;
    
    /** Stores the JTextField type settings */
    protected ComponentDocument.Component resourceElementType;
    
    /** Stores the value ot the Swing element component */
    protected JComponent elementComponent;
    
    /** Holds value of property tabIndex. */
    private int tabIndex;
    
    /**
     * Creates a new instance of the Element Component
     * @param resourceElement details specyfied by the resource element object
     */
    public ElementComponent(ElementDocument.Element resourceElement){
        log.debug("ElementComponent");
        this.resourceElement = resourceElement;
        //get the element type
        this.resourceElementType = ComponentFactory.getComponentById(resourceElement.getComponent().getId() );
    }
    
    /**
     * The method is used to initialize the java component
     * and set the component properties, which are taken from
     * the resource element description
     */
    protected void initElement(){
        //set element name
        String name = getComponentName();
        log.debug("Initializing component:"+name);
        elementComponent.setName(name);
        setFont(ComponentKeys.RESOURCE_VALUE_FONT);    //font
        //set tooltip
        setToolTipText(getToolTipText());
        //set enabled
        setEnabled( isEnabled());
        //set visible
        setVisible(isVisible());
    }
    
    
    /**
     * Get the resource element
     */
    public  ElementDocument.Element getResourceElement(){
        return resourceElement;
    }
    
    /**
     * Get the concrete java component which represents this element component
     * @return JComponent which represents the this element component
     */
    public JComponent getConcreteJavaComponent() {
        return this.elementComponent;
    }
    
    /**
     * Get the component name, which is put together form
     * the elment name and Value string
     * @return
     */
    protected String getComponentName(){
        return this.resourceElement == null ? "Bad Id. ResourceElement is NULL" : new StringBuffer().append(this.resourceElement.getName()).append("Value").toString();
    }
    
    /**
     * Get the component display name
     * @return
     */
    protected String getDisplayName(){
        return this.resourceElement == null ? "Bad Id. ResourceElement is NULL" : this.resourceElement.getDisplayName();
    }
    
    
    /**
     * Tells whether the component will be enabled or not.
     * A component that is enabled may respond to user input,
     * while a component that is not enabled cannot respond to user input.
     * Some components may alter their visual representation when they are
     * disabled in order to provide feedback to the user that they cannot take input.
     * @return true if this component is enabled, false otherwise
     */
    public boolean isEnabled(){
        return resourceElementType.getEnabled();
    }
    
    /**
     * Sets whether or not this component is enabled. A component that is enabled may respond to user input, while a component that is not enabled cannot respond to user input. Some components may alter their visual representation when they are disabled in order to provide feedback to the user that they cannot take input.
     * @param enabled true if this component should be enabled, false otherwise
     */
    public void setEnabled(boolean enabled){
        //set resource as enabled
        resourceElement.getComponent().setEnabled(enabled);
        //set component as enabled
        elementComponent.setEnabled(enabled);
    }
    
    
    
    /**
     * Tells whether the component will be visible or not
     * @return
     */
    protected boolean isVisible(){
        if(resourceElement.getComponent().isSetVisible()){
            return resourceElement.getComponent().getVisible();
        }
        return resourceElementType.getVisible();
    }
    
    
    /**
     * Makes the component visible or invisible. Overrides Component.setVisible.
     * @param visible  true to make the component visible; false to make it invisible
     */
    public void setVisible(boolean visible){
        //set resource as enabled
        resourceElement.getComponent().setVisible(visible);
        //set component as enabled
        elementComponent.setVisible(visible);
    }
    
    
    /**
     * Returns the tooltip string that has been set with setToolTipText.
     * @return the text of the tool tip
     */
    protected String getToolTipText(){
        if(resourceElement.getComponent().isSetToolTipText()){
            return resourceElement.getComponent().getToolTipText();
        }
        return null;
    }
    
    /**
     * Registers the text to display in a tool tip. The text displays when the cursor lingers over the component.
     * @param text the string to display; if the text is null, the tool tip is turned off for this component
     */
    public void setToolTipText(String text){
        if(FormTools.isValidateFormStringNull(text)){
            return;
        }
        //set resource as enabled
        resourceElement.getComponent().setToolTipText(text);
        //set component as enabled
        elementComponent.setToolTipText(text);
    }
    
    
    /**
     * Sets the component background color
     * @param new color
     */
    public void setBackground(Color newColor){
        elementComponent.setBackground(newColor);
    }
    
    /**
     * Sets the font for this component.
     * @param font the desired Font for this component
     */
    public void setFont(Font font){
        elementComponent.setFont(font);
    }
    
    /**
     * Gets font for this component
     * @return this component's font; if a font has not been set for this component, the font of its parent is returned
     */
    public Font getFont(){
        return elementComponent.getFont();
    }
    
    
    /**
     * Get the list of the model item names
     * @return
     */
    protected ItemDocument.Item[] getModelItems(){
        if(this.resourceElementType.getModelDataAttached()){
            //check the external link
           /* tbd FIXME
            if(this.resourceElement.getComponent().getModelData().getIsExternal()){
                //todo DO NOT IGNORE that exception
                try{
                    ExternalDocument.External externalModel = this.resourceElement.getComponent().getModelData().getExternal();
                    SSARemote vlabRemote = new SSARemoteImpl();
                    SessionValue sessionVal = SessionFactory.getSessionValue();
                    return vlabRemote.executeGenericQuery(externalModel.getTableName(),
                            externalModel.getAttributes(), String.valueOf(sessionVal.getUserId()), externalModel.getOrderBy());
                }catch(Exception e){
                    log.error("Generic query error"+e.getMessage());
                    return null;
                }
            }*/
            return resourceElement.getComponent().getModelData().getItemArray();
        }
        return null;
    }
    
    
    
    /**
     * Get the list of the out item
     * @return
     */
    protected ItemDocument.Item[] getOutItems(){
        if(resourceElement.getOut().sizeOfItemArray() <= 0){
            return null;
        }
        return resourceElement.getOut().getItemArray();
    }
    
    
    /**
     * Gets the component model class name
     * @return
     */
    public String  getModelClassName(){
        String model = "";
        if(resourceElement.getComponent().isSetModel()){
            model = resourceElement.getComponent().getModel();
        } else {
            if(resourceElementType.isSetModel()){
                model = resourceElementType.getModel();
            }
        }
        return model;
    }
    
    
    /**
     * Getter for property tabIndex.
     * @return Value of property tabIndex.
     */
    public int getTabIndex() {
        return this.tabIndex;
    }
    
    /**
     * Setter for property tabIndex.
     * @param tabIndex New value of property tabIndex.
     */
    public void setTabIndex(int tabIndex) {
        this.tabIndex = tabIndex;
    }
    
    
    /**
     * Get the layout constraint for the given element component
     * @return
     */
    public ConstraintsDocument.Constraints getLayoutConstraints(){
        return resourceElement.getConstraints();
    }
    
    
    public void addChangeListener(ArrayList grp1, ArrayList grp2) {
        
    };
    
}