/*
 * ElementFactory.java
 *
 * Created on 17 czerwiec 2004, 11:05
 */

package pl.psnc.vlab.ssa.gui.resource.components;

import pl.psnc.vlab.ssa.util.component.ComponentHelper;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.util.FormTools;
import pl.psnc.vlab.rds.ElementDocument;
//schema
import pl.psnc.vlab.cds.ItemDocument;


/**
 * The abstract super class for building the
 * java components from the resource elements.
 * It contains a set of common util methods for the text components the i.e. for setting
 * the java component properties (name, tooltip, etc.)
 * @author  Dominik Stoklosa
 * @email  osa@man.poznan.pl
 */
public abstract class ElementTextComponent extends ElementComponent {
    
    /** Instance of logger */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    
    /**
     * Creates a new instance of the Element Component
     * @param resourceElement details specyfied by the resource element object
     */
    public ElementTextComponent(ElementDocument.Element resourceElement){
        super(resourceElement);
        log.debug("ElementTextComponent");
    }
    
    /**
     * The method is used to initialize the java component
     * and set the component properties, which are taken from
     * the resource element description
     */
    protected void initElement(){
        //call parent to initialize its components
        super.initElement();
        //set element name
        String name = getComponentName();
        log.debug("Initializing component:"+name);
        //set editable
        setEditable(isEditable());
        //set text
        setText(getText());
        //set document
        setDocument(getDocument());
    }
    
    
    /**
     * Tells whether the component will be editable or not.
     * Set to true by default
     * @return
     */
    public boolean isEditable(){
        if(resourceElement.getComponent().isSetEditable()){
            return resourceElement.getComponent().getEditable();
        } else if(resourceElementType.isSetEditable()){
            return resourceElementType.getEditable();
        }
        return true;
    }
    
    /**
     * Sets the specified boolean to indicate whether or not this TextComponent should be editable. A PropertyChange event ("editable") is fired when the state is changed.
     * @param editable
     */
    public void setEditable(boolean editable){
        //set resource as enabled
        resourceElement.getComponent().setEditable(editable);
        //set component as enabled
        doCast().setEditable(editable);
    }
    
    
    /**
     * Returns the text contained in this TextComponent. If the underlying document is null, will give a NullPointerException.
     * @return the text from this component
     */
    public String getText(){
        String value = doCast().getText();
        if(!FormTools.isValidateFormStringNull(value)){
            return value;
        } 
        ItemDocument.Item items[] = getOutItems();
        int size = items != null ? items.length : 0;
        if(size >= 1){
            return items[0].getStringValue();
        } else if(resourceElement.getComponent().isSetText()){
            return resourceElement.getComponent().getText();
        }
        return "";
    }
    
    
    /**
     * Sets the text of this TextComponent  to the specified text. If the text is null  or empty, has the effect of simply deleting the old text. When text has been inserted, the resulting caret location is determined by the implementation of the caret class.
     * @param text the new text to be set
     */
    public void setText(String text){
        //set resource as enabled
        resourceElement.getComponent().setText(text);
        //set component as enabled
        doCast().setText(text);
    }
    
    
    /**
     * Fetches the model associated with the editor. This is primarily for the UI to get at the minimal amount of state required to be a text editor. Subclasses will return the actual type of the model which will typically be something that extends Document
     * @return
     */
    public String getDocument(){
        String document = null;
        if(resourceElement.getComponent().isSetDocument()){
            document = resourceElement.getComponent().getDocument();
        } else if(resourceElementType.isSetDocument()){
            document = resourceElementType.getDocument();
        }
        return document;
    }
    
    
    
    /**
     * Associates the editor with a text document. The currently registered factory is used to build a view for the document, which gets displayed by the editor after revalidation. A PropertyChange event ("document") is propagated to each listener.
     * @param doc
     */
    public void setDocument(String doc){
        //set resource document
        resourceElement.getComponent().setDocument(doc);
        //set component document
        Object documentInstance = ComponentHelper.loadClass(doc);
        if(documentInstance == null || !(documentInstance instanceof Document)){
            return ;
        }
        doCast().setDocument((Document) documentInstance);
    }
    
    
    
    /**
     * Cast the element component to the JTextComponent interface
     * @return element component as an JTextComponent
     */
    private JTextComponent doCast(){
        return (JTextComponent) elementComponent;
    }
    
}