/*
 * BytesConversion.java
 *
 * Created on 18 marzec 2004, 14:44
 */

package pl.psnc.vlab.ssa.util;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.StringTokenizer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pl.psnc.vlab.conf.ConfigFactory;
import pl.psnc.vlab.exception.VlabException;
import pl.psnc.vlab.remote.util.BytesConversion;
import pl.psnc.vlab.util.FormTools;


/**
 * This helper class is used to decode or encode the data which is sent
 * between the application and the VLab System. The class contains 
 * conversions specific to the Scenario Submission Application
 * @author  osa
 * @email osa@man.poznan.pl
 */
public class SSABytesConversion extends BytesConversion {
    
    /** The size (in bytes) of the integer variable */
    private static int INT_SIZE = 4;
    
    /** Logger instance */
    private Log log = LogFactory.getLog(this.getClass().getName());
    
    /** Nucleus quantity */
    private static final int NUCLEUS_QUANTITY = 6;
        
    
    /**
     * Converts the Dynamic Scenario object into an array of bytes. The
     * Scenario consists of two parts: the jgx file - which is the graph
     * representation of the scenario (nodes and connections between those nodes)
     * and the list of resources. Each resource represents one node in the graph.
     * @return array of bytes
     * @param jgxFile
     * @param resources
     */
    public  byte[] scenarioDescriptionToBytes(String jgxFile, String resources){
        log.debug("<scenarioDescriptionToBytes>");
        byte jgxB[] = toUtf(jgxFile).getBytes();
        byte resourcesB[] =  resources.getBytes();  // toUtf(resources).getBytes();
        int jgxSize = jgxB.length;
        int resSize = resourcesB.length;
        log.debug("JgxSize="+jgxSize +" ResSize="+resSize);
        log.debug("Converted DMS="+new String(resourcesB));
        ByteBuffer byteBuffer = ByteBuffer.allocate(jgxSize + resSize + INT_SIZE + INT_SIZE);
        byteBuffer.putInt(jgxSize);                         //jgx XML file size
        byteBuffer.putInt(resSize);                         //resources XML file size
        byteBuffer.put(jgxB );                              //jgx XML file
        byteBuffer.put(resourcesB);                         //resources XML file
        log.debug("</scenarioDescriptionToBytes>");
        return byteBuffer.array();
    }
    
    
    
    /**
     * Converts the string value into the Utf representation
     * @param value string value to be converted
     * @return
     */    
    public String toUtf(String value){
        if( FormTools.isValidateFormStringNull(value)){
            return null;
        }
        try{
            byte b[] = value.getBytes("UTF-8");
            return new String(b);
        }catch(Exception e){
            e.printStackTrace();
        }
        return "UTF bad value";
    }
    
   
    
    /**
     * Convers the struct genericSelectQuery into an array of bytes. The struct
     * looks as the following:
     *  struct genericSelectQuery
     *   {
     *       char tableName[128];           //name of the database table
     *       int  paramCount;               //number of parameters
     *       char params[256];              //parameters list
     *       char orderBy[128];             //order by (for sql ORDER BY)
     *    };
     * @param tableName database table name
     * @param attributes the list of atributes
     * @param condition sql where clause
     * @param orderBy sql order by
     * @return
     */
    public byte[] encodeGenericQuery(String tableName, String attributes,
    String condition, String orderBy) throws VlabException{
        ByteBuffer byteBuffer = ByteBuffer.allocate(SSARemoteKeys.GENERIC_QUERY_REQUEST_SIZE);        
        
        byteBuffer.put(toBytes(tableName, 128));                                        //table name
        int tokenCount = new StringTokenizer(attributes.trim(), ",").countTokens();
        byteBuffer.putInt(tokenCount);                                                  //number of attributes
        byteBuffer.put(toBytes(attributes, 256));                                       //attributes
        byteBuffer.put(toBytes(orderBy, 128));                                          //order by clause
        return byteBuffer.array();
    }
}
